#include <cmath>
#include <iostream>
#include <string>


/************************************/
// Declarations
/************************************/

//! Returns `number` * (2 ^ `exponent`) 
int TimesPowerOf2(int number, int exponent);

//! Round `x` to the nearest integer.
int RoundAsInt(double x);

//! Function for error handling. We will see later how to fulfill the same functionality more properly.
//! Don't bother here about [[noreturn]] - it's just a keyword to silence a possible warning telling
//! the program may not return at the calling site (which is definitely the case here as there is a 
//! std::exit() called in the function).
[[noreturn]] void Error(std::string explanation);

//! Display the approximation of the given argument that does not exceed the chosen maximum numerator.
void DisplayPowerOf2Approx(int max_numerator, double value);

//! Helper function that computes numerator and denominator.
//! You're not obliged to use a function, but this way you enforce the Don't Repeat Yourself (DRY) principle!
//! We'll see much later that it is typically the kind of function that could be put in an anonymous namespace.
void HelperDisplayPowerOf2Approx(double value, int exponent, int& numerator, int& denominator);



/************************************/
// Definitions
/************************************/

int TimesPowerOf2(int number, int exponent)
{
    // Very crude implementation that is not safe enough - we'll remedy this later...
    while (exponent > 0)
    { 
        number *= 2; 
        exponent -= 1; 
    }
    while (exponent < 0)
    { 
        number /= 2;
        exponent += 1 ; 
    }
    
    return number;
}
    

int RoundAsInt(double x)
{
    // Very crude implementation that is not safe enough - we'll remedy this later...
    return static_cast<int>(std::round(x));
}


[[noreturn]] void Error(std::string explanation)
{
    std::cout << "ERROR: " << explanation << std::endl;
    exit(EXIT_FAILURE);
}


void HelperDisplayPowerOf2Approx(double value, int exponent, int& numerator, int& denominator)
{
    denominator = TimesPowerOf2(1, exponent);   
    numerator = RoundAsInt(value * denominator);
}


void DisplayPowerOf2Approx(int max_numerator, double value)
{
    int numerator {}, denominator{}, exponent {};
    
    do
    {
        // I used here the prefix increment '++exponent' but you may put it on a separate line if you're not 
        // comfortable with it.
        HelperDisplayPowerOf2Approx(value, ++exponent, numerator, denominator); 
    }
    while (numerator < max_numerator);
    // After the while loop we have numerator > max_numerator,  
    // hence we need to update the fraction using the previous exponent with --exponent.
    HelperDisplayPowerOf2Approx(value, --exponent, numerator, denominator);
    
    double double_quotient = static_cast<double>(numerator) / denominator; 
        
    std::cout << "[With numerator < "<< max_numerator << "]: " << value << " ~ " << double_quotient << " (" << numerator << 
        " / 2^" << exponent << ')' << std::endl;
}


/************************************/
// Main function
/************************************/

// [[maybe_unused]] is a C++ 17 keyword to indicate we're fully aware the variable may not be used.
int main([[maybe_unused]] int argc, [[maybe_unused]] char** argv)
{        
    DisplayPowerOf2Approx(15, 0.65);
    DisplayPowerOf2Approx(255, 0.65);

    DisplayPowerOf2Approx(15, 0.35);
    DisplayPowerOf2Approx(255, 0.35);
    
    return EXIT_SUCCESS;
}

